import { Buttons, Chat } from "whatsapp-web.js";

interface iUser {
  fromNumber: string;
  body: string;
  step: number;
  createdAt: Date;
}

interface iConversation {
  fromNumber: string;
  body: string;
  chat: any;
}

// Opções de pergunta
const BotMessage = [
  {
    step: 0,
    message: "Olá, meu nome é Eduzinho e vou te ajudar em sua jornada!",
  },
  {
    step: 1,
    message: "Para isso me ajude respondendo o que deseja fazer",
    buttons: [
      { body: "Quero receber notificações por WhatsApp" },
      { body: "Preciso de ajuda" },
    ],
  },
  {
    step: 2,
    message: "Para isso me ajude respondendo o que deseja fazer",
    buttons: [
      { body: "Não quero mais receber notificações por WhatsApp" },
      { body: "Preciso de ajuda" },
    ],
  },
  {
    step: 3,
    message: "Logo um de nossos agentes entrará em contato",
  },
  {
    step: 4,
    message: "Não enviaremos mais notificações por WhatsApp",
  },
  {
    step: 5,
    message:
      "Tudo pronto, agora você vai começar a receber todas as atualizações mais importante também por WhatsApp",
  },
];

// Strings
const success_anwser =
  "Tudo certo por aqui, quando tiver notificações importantes você receberá por aqui também";
const failed_onRegister_anwser = "Algo deu errado, tente novamente mais tarde";
const failed_anwser = "Opção inválida";
const await_anwser =
  "Em breve um atendente dará continuidade com o seu atendimento";

const userList = [];

// Adiciona um usuário na lista de atendimento
const setUserToList = ({ ...user }: iUser) => {
  userList.push(user);
};

// Remove um usuário da lista de atendimento
const removeUserFromList = (fromNumber: string) => {
  let indexList = userList.findIndex(
    (user: iUser) => user.fromNumber === fromNumber
  );

  userList.splice(indexList, 1);
};

// Busca um usuário da lista de atendimento
const getUserFromList = (fromNumber: string): iUser => {
  let user = userList.find((user: iUser) => user.fromNumber === fromNumber);

  if (!user) return undefined;

  return user;
};

// Verifica se esse usuário já tem o WhatsApp ativado como opção de notificação
const getUserFromAppEdu = async (phone: number) => {
  await Promise.resolve(setTimeout(() => {}, 2000));

  return false;
};

// Atualiza esse usuário para ativar ou desativar a opção de envio por WhatsApp
const setUserToAppEdu = async (phone: number) => {
  await Promise.resolve(setTimeout(() => {}, 2000));

  return true;
};

const conversation = async ({ ...conversation }: iConversation) => {
  const findUser = getUserFromList(conversation.fromNumber);

  const phone = parseInt(conversation.fromNumber.split("@")[0]);

  const hasWhatsAppNotification = await getUserFromAppEdu(phone);

  if (findUser === undefined) {
    await global.WhatsappUtils.sendMessage(
      conversation.fromNumber,
      BotMessage[0].message
    );

    if (!hasWhatsAppNotification) {
      const buttons = new Buttons(
        BotMessage[1].message,
        [{ body: "teste 01" }, { body: "teste 02" }],
        "olá",
        "mundo"
      );

      await global.WhatsappUtils.sendMessage(conversation.fromNumber, buttons);
    }

    setUserToList({
      fromNumber: conversation.fromNumber,
      body: conversation.body,
      step: 1,
      createdAt: new Date(),
    });
  } else {
    let user = getUserFromList(conversation.fromNumber);

    if (user.step === 1) {
      if (
        BotMessage[1].buttons.find(
          (option) => option.body === conversation.body
        )
      ) {
        if (conversation.body === BotMessage[1].buttons[0].body) {
          const data = await setUserToAppEdu(phone);

          if (data) {
            await global.WhatsappUtils.sendMessage(
              conversation.fromNumber,
              success_anwser
            );
          } else {
            await global.WhatsappUtils.sendMessage(
              conversation.fromNumber,
              failed_onRegister_anwser
            );
          }

          removeUserFromList(conversation.fromNumber);
        } else {
          await conversation.chat.markUnread();
          await global.WhatsappUtils.sendMessage(
            conversation.fromNumber,
            await_anwser
          );
        }
      } else {
        await global.WhatsappUtils.sendMessage(
          conversation.fromNumber,
          failed_anwser
        );
      }
    }
  }
};

export { conversation };
