import dayjs from "dayjs";
import "dayjs/locale/pt-br"; // carregar sob demanda
import { jsPDF } from "jspdf";
import puppeteer from "puppeteer";
import { Request, Response } from "express";

dayjs.locale("pt-br");

import fonts from "../constant/fonts";
import { base64 } from "../../public/images/record_book/brasao";

const A4 = 210;
const MT = 30;
const MB = 20;
const MR = 20;
const ML = 30;

const center = (name: string, doc: any, startAt?: number, endAt?: number) => {
  let nameWidth = doc.getTextWidth(name);
  if (!startAt && !endAt) {
    let spaces = A4 - nameWidth;
    return spaces / 2;
  } else {
    let width = endAt - startAt;
    let spaces = width - nameWidth;
    return startAt + spaces / 2;
  }
};

const retangle = (
  doc: any,
  x: number | "center",
  y: number,
  width: number,
  height: number,
  style: string
) => {
  // Cria um retangulo para a identificação
  doc.setFillColor(242, 242, 242);
  doc.rect(x === "center" ? (A4 - width) / 2 : x, y, width, height, style);

  return doc;
};

interface iCover {
  school_name: string;
  course: string;
  curricular_component: string;
  year: string;
  shift: string;
  degree: string;
  class_year: string;
  class: string;
}

interface iContent {
  first_bimester: string;
  second_bimester: string;
  third_bimester: string;
  fourth_bimester: string;
}

interface iAttendance {
  teacher: string;
  bimester: number;
  working_days: number;
  worked_days: number;
  data: {
    number: number;
    name: number;
    status: number;
    attendances: {
      date: string;
      attended: boolean;
    }[];
    justified_attendances: number;
    grades: {
      subject: string;
      grade: string;
    }[];
  }[];
}

const cover = ({ ...data }: iCover) => {
  const doc = new jsPDF();

  // Adiciona as fonts
  doc.addFileToVFS(fonts[0].path, fonts[0].font);
  doc.addFileToVFS(fonts[1].path, fonts[1].font);
  doc.addFont(fonts[0].path, "Roboto", "normal");
  doc.addFont(fonts[1].path, "RobotoBlack", "normal");

  doc.addImage(base64, "PNG", (A4 - 55) / 2, 15, 55, 63.9);

  doc.setFont("RobotoBlack");
  doc
    .setFontSize(15)
    .text(
      "SECRETARIA DE ESTADO DA EDUCAÇÃO",
      center("SECRETARIA DE ESTADO DA EDUCAÇÃO", doc),
      90
    );

  retangle(doc, "center", 110, 170, 5.6, "F");

  doc.setFont("RobotoBlack");
  doc.setFontSize(8).text("IDENTIFICAÇÃO DA ESCOLA", 22.5, 114);

  doc.setFont("Roboto");
  doc
    .setFontSize(20)
    .text(data.school_name, center(data.school_name, doc), 125);

  doc.setFont("RobotoBlack");
  doc
    .setFontSize(40)
    .text(`Diário de classe`, center(`Diário de classe`, doc), 165);

  retangle(doc, "center", 200, 170, 5.6, "F");
  doc.setFontSize(8).text("CURSO: ", 22.5, 204);

  doc.setFont("Roboto");
  doc.setFontSize(20).text(data.course, center("FUNDAMENTAL", doc), 215);

  doc.setFont("RobotoBlack");
  retangle(doc, "center", 230, 170, 5.6, "F");
  doc.setFontSize(8).text("COMPONENTE CURRICULAR", 22.5, 234);

  doc.setFont("Roboto");
  doc
    .setFontSize(20)
    .text(data.curricular_component, center("ARTES", doc), 245);

  doc.setFont("RobotoBlack");
  retangle(doc, "center", 260, 170, 5.6, "F");

  // Titulos
  doc.setFontSize(8).text("ANO: ", 22.5, 264);
  doc.setFontSize(8).text("TURNO: ", 56.5, 264);
  doc.setFontSize(8).text("GRAU: ", 90.5, 264);
  doc.setFontSize(8).text("ANO: ", 124.5, 264);
  doc.setFontSize(8).text("TURMA: ", 158.5, 264);

  // Linhas
  doc.line(55, 260, 55, 275);
  doc.line(89, 260, 89, 275);
  doc.line(123, 260, 123, 275);
  doc.line(157, 260, 157, 275);

  // Conteúdo
  doc.setFont("Roboto");
  doc.setFontSize(15).text(data.year, center(data.year, doc, 22.5, 55), 273);
  doc.setFontSize(15).text(data.shift, center(data.shift, doc, 55, 89), 273);
  doc.setFontSize(15).text(data.degree, center(data.degree, doc, 89, 123), 273);
  doc
    .setFontSize(15)
    .text(data.class_year, center(data.class_year, doc, 123, 157), 273);
  doc.setFontSize(15).text(data.class, center(data.class, doc, 157, 190), 273);

  doc.save("cover.pdf");
};

const content = ({ ...data }: iContent): string => {
  console.log(data.first_bimester.match(/\d{5}\.\r?\n\d{5}/));
  return `<!DOCTYPE html>
  <html lang="en">
    <head>
      <meta charset="UTF-8" />
      <meta name="viewport" content="width=device-width, initial-scale=1.0" />
      <title>Conteúdo</title>
  
      <style>
        * {
          font-family: Arial, Helvetica, sans-serif;
        }
        body {
          background-color: black;
          display: flex;
          justify-content: center;
        }
        .content {
          padding: 10mm;
          background-color: white;
          width: 210mm;
          height: 297mm;
        }
        .title {
          text-align: center;
          font-size: 12pt;
          border-style: solid;
          border-color: gray;
          border-top-width: 1mm;
          border-bottom-width: 1mm;
          border-left: 0;
          border-right: 0;
          padding: 2mm 0mm;
        }
        .subtitle {
          font-size: 8pt;
          text-align: center;
          margin-bottom: 2mm;
        }
        .content-lines {
          margin-top: 10mm;
        }
  
        .content-text {
          width: 180mm;
          top: 42mm;
          line-height: 5.52mm;
          position: absolute;
        }
  
        p {
          margin: 0;
          font-size: 10pt;
        }
  
        hr {
          margin-top: 5mm;
        }
      </style>
    </head>
    <body>
      <div class="content">
        <h2 class="subtitle">1º BIMESTRE</h2>
        <h1 class="title">
          RESUMO DO CONTEÚDO PROGRAMÁTICO E DAS ATIVIDADES DESENVOLVIDAS
        </h1>
  
        <div class="content-lines">
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
          <hr />
        </div>
  
        <div class="content-text">
        ${data.first_bimester}
        </div>
      </div>
    </body>
  </html>  
  `;
};

const groupMonths = (data: any) => {
  let tempUniDays = [];

  data?.map(({ data: dates }) => {
    dates?.map((student) => {
      student?.attendances.map((attendance) => {
        if (!tempUniDays.includes(attendance.date)) {
          tempUniDays.push(attendance.date);
        }
      });
    });
  });

  let result = Object.values(
    tempUniDays.reduce((r, date) => {
      const [year, month, day] = date.substr(0, 10).split("-");
      const key = `${year}_${month}`;
      r[key] = r[key] || { month, year, dates: [] };
      r[key].dates.push(day);
      return r;
    }, {})
  );

  result = result.sort((a: any, b: any) => {
    if (a.month < b.month) return -1;
    if (a.month > b.month) return 1;
    return 0;
  });

  tempUniDays = tempUniDays.sort((a: any, b: any) => {
    if (
      parseInt(dayjs(a).format("YYYYMMDD")) <
      parseInt(dayjs(b).format("YYYYMMDD"))
    )
      return -1;
    if (
      parseInt(dayjs(a).format("YYYYMMDD")) >
      parseInt(dayjs(b).format("YYYYMMDD"))
    )
      return 1;

    return 0;
  });

  // let tempDates = [];

  // new Array(77).fill(0).map((item, index) => {
  //   console.log(index);
  //   let tempDate = dayjs("2023-02-30").add(index, "days").format("YYYY-MM-DD");

  //   tempDates.push({
  //     date: tempDate,
  //     attended: false,
  //   });
  // });

  // console.log(JSON.stringify(tempDates));

  return { allDays: tempUniDays, formatDays: result };
};

const attendance = (data: Array<iAttendance>): string => {
  let daysToShow = groupMonths(data);

  return `<!DOCTYPE html>
  <html lang="en">
    <head>
      <meta charset="UTF-8" />
      <meta name="viewport" content="width=device-width, initial-scale=1.0" />
      <title>Faltas</title>
  
      <style>
        html {
          -webkit-print-color-adjust: exact;
        }
        * {
          font-family: Arial, Helvetica, sans-serif;
        }

        table, th, td {
          border: 1px solid black;
          border-collapse: collapse;
        }

        .white-color {
          color: white;
        }
        .text-8 {
          font-size: 8pt;
        }
  
        .text-bold {
          font-weight: bold;
        }
        .text-regular {
          font-weight: 400;
        }
        .p-1 {
          padding: 0.2cm;
        }
        .px {
          padding: 0.2cm;
        }
        .text-italic {
          font-style: italic;
        }
        .center {
          text-align: center;
        }
        .width-6{
          width: 1000px
        }
    
        
      </style>
    </head>
    <body>
      <table border="1px">
        <thead>
          <th colspan="2">
            <span class="white-color text-8">Nº CH</span>
          </th>
          <th bgcolor="#f1f1f1">
            <span class="text-italic text-regular text-8 px">MESES</span>
          </th>
          ${daysToShow?.formatDays.map((day: any) => {
            return `<th colspan="${day.dates.length}">
          <span class="px">${dayjs()
            .month(day.month - 1)
            .format("MMMM")
            .toUpperCase()}</span>
          </th>`;
          })}
          
        </thead>
        <tbody style="border: 1px solid black;">
          <tr>
            <td class="p-1" bgcolor="#3f3f3f">
              <span class="white-color text-8">Nº CH</span>
            </td>
            <td style="width: 2000px"><span class="text-bold p-1">NOME DOS ALUNOS</span></td>
            <td class="center" bgcolor="#f1f1f1">
              <span class="text-italic text-regular text-8 px">DIAS</span>
            </td>
            ${daysToShow?.allDays.map(
              (day: any) => `<td>
            <span  class="px">${dayjs(day).format("DD")}</span>
            </td>`
            )}
          </tr>

          ${data?.map(({ data: dates }) => {
            return dates?.map((student) => {
              return `<tr>
                <td>${student.number}</td>
                <td>${student.name}</td>
                <td class="center">${student.number}</td>
                ${student?.attendances?.map((att) => {
                  return `<td class="center" bgcolor="${
                    att.attended ? "white" : "red"
                  }"><span>${att.attended ? "C" : "F"}</span></td>`;
                })}
                </tr>`;
            });
          })}

        </tbody>
      </table>
    </body>
  </html>`;
};

const PdfController = {
  create: async (req: Request, res: Response) => {
    // Gera a capa
    // cover({ ...req.body.cover });

    // Fera o conteúdo
    // content({ ...req.body.content })

    // Gera os conteúdos
    const browser = await puppeteer.launch({});
    const page = await browser.newPage();

    console.log(attendance(req.body.attendance));

    await page.setContent(attendance(req.body.attendance));

    const pdf = await page.pdf({
      path: "attendance.pdf",
      format: "A4",
      landscape: true,
    });

    await browser.close();

    res.send("ok");
  },
};

export default PdfController;
