import { NextFunction, Request, Response } from "express";
import { object, string, number, ValidationError } from "yup";
import { images } from "../../constant/image";

const Validators = {
  requestRecipient: object({
    recipient: number()
      .typeError(
        "Você deve informar um número de whatsapp valido para o destinatário"
      )
      .required("Você deve um destinátario"),
  }),
  sendText: object({
    message: string().required("Você deve informar sua mensagem"),
  }),
  sendImage: object({
    message: string(),
    image: object({
      name: string()
        .required("Informe qual a imagem")
        .test({
          name: "hasImage",
          test: (value) => {
            return images.hasOwnProperty(value);
          },
          message: "Imagem invalida",
        }),
      alt: string(),
    }),
  }),
};

const WhatsappValidator = {
  requestRecipient: async (req: Request, res: Response, next: NextFunction) => {
    await Validators.requestRecipient.validate(req.body);
    next();
  },
  sendText: async (req: Request, res: Response, next: NextFunction) => {
    await Validators.sendText.validate(req.body);
    next();
  },
  sendImage: async (req: Request, res: Response, next: NextFunction) => {
    await Validators.sendImage.validate(req.body);
    next();
  },
};

export default WhatsappValidator;
